from typing import Optional
from uuid import uuid4

from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, SubscriptionLifecycle, UUIDstr
from orchestrator.workflow import done, init, step, workflow
from orchestrator.workflows.steps import resync, set_status, store_process_subscription
from orchestrator.workflows.utils import wrap_create_initial_input_form

from products.product_types.trunk import TrunkInactive, TrunkProvisioning


def initial_input_form_generator(product_name: str) -> FormGenerator:
    class CreateTrunkForm(FormPage):
        class Config:
            title = product_name

        trunk_name: str
        geant_s_sid: Optional[str]

    user_input = yield CreateTrunkForm

    return user_input.dict()


def _provision_in_gap(trunk_name: str) -> int:

    return abs(hash(trunk_name))


@step("Create subscription")
def create_subscription(product: UUIDstr) -> State:
    subscription = TrunkInactive.from_product_id(product, uuid4())

    return {"subscription": subscription, "subscription_id": subscription.subscription_id}


@step("Initialize subscription")
def initialize_subscription(subscription: TrunkInactive, trunk_name: str, geant_s_sid: str) -> State:
    subscription.trunk.trunk_name = trunk_name
    subscription.trunk.geant_s_sid = geant_s_sid
    subscription.description = f"Trunk {trunk_name}"
    subscription = TrunkProvisioning.from_other_lifecycle(subscription, SubscriptionLifecycle.PROVISIONING)

    return {"subscription": subscription}


@step("Provision trunk")
def provision_trunk(subscription: TrunkProvisioning, trunk_name: str) -> State:
    trunk_id = _provision_in_gap(trunk_name)
    subscription.trunk.trunk_id = trunk_id

    return {"subscription": subscription}


@workflow(
    "Create trunk",
    initial_input_form=wrap_create_initial_input_form(initial_input_form_generator),
    target=Target.CREATE,
)
def create_trunk():

    return (
        init
        >> create_subscription
        >> store_process_subscription(Target.CREATE)
        >> initialize_subscription
        >> provision_trunk
        >> set_status(SubscriptionLifecycle.ACTIVE)
        >> resync
        >> done
    )
